#pragma once

//amount of equations for the solver
#define NEQN 54
// #define NEQN 12

//structs for the simulation
#include "SimulationStructs.h"

#include "CoreMinimal.h"
#include "GameFramework/Actor.h"
#include "Simulation.generated.h"

UENUM(BlueprintType)
enum class SolverEnum : uint8
{
	EULER					UMETA(DisplayName = "Euler"),
	EULER_CROMER			UMETA(DisplayName = "Euler-Cromer"),
	RUNGE_KUTTA				UMETA(DisplayName = "Runge-Kutta"),
	RUNGE_KUTTA_FEHLBERG	UMETA(DisplayName = "Runge-Kutta-Fehlberg"),
};

UCLASS()
class GRAVSIMULATION_API ASimulation : public AActor
{
	GENERATED_BODY()

	
public:	
	// Sets default values for this actor's properties
	ASimulation();

protected:
	virtual void BeginPlay() override;
	virtual void EndPlay(const EEndPlayReason::Type EndPlayReason) override;

public:	
	// Called every frame
	virtual void Tick(float DeltaTime) override;

//variables editable from unreal editor
public:
	UPROPERTY(EditAnywhere)
		TArray<class AActor*> actorRefs;
	UPROPERTY(EditAnywhere)
		float timeScaleConstant;
	UPROPERTY(EditAnywhere)
		SolverEnum solver;
	UPROPERTY(EditAnywhere)
		int simLoopsPerTick;
	UPROPERTY(EditAnywhere)
		bool useFixedTimeStep;
	UPROPERTY(EditAnywhere)
		float fixedTimeStep;

//private methods
private:
	void InitializeBodies();
	void RealSolarSystem();
	void SingleBodyTest();
	void InitializeSolver();
	void Euler(double DeltaTime);
	void EulerCromer(double DeltaTime);
	void RungeKutta(double DeltaTime);
	FVectorDouble CalculateAcceleration(FVectorDouble position, int bodyIndex);
	void RungeKuttaFehlberg(double DeltaTime);
	void RecordSnapshot();
	void ExportPlanetPositions(FString planetName, int planetIndex);
	void ExportSingleBodyErrors();
	void ValidateRealSystem();
	double CalculateValidationError(FVectorDouble simPos, FVectorDouble realPos);
	void ExportValidationErrors(TArray<double> errors, FString name);

//private variables
private:
	TArray<FBody> bodies;
	TArray<FSnapshot> snapshots;
	int tickCount;
	double simTime;
	double timeScale;

//solver parameters for runge-kutta-fehlberg
private:
	double y[NEQN];
	double dydt[NEQN];
	double t;
	double tout;
	double relerr;
	double abserr;
	int flag;
};
