#include "SimulationRHS.h"

#define NR_OBJECTS 9
// #define NR_OBJECTS 2

//parameters
double bodyGMs[NR_OBJECTS];
double timeScaling;

//used to set the equations parameters
void SetParams(double pBodyGMs[], double pTimeScaling) 
{
	for (int i = 0; i < NR_OBJECTS; i++) bodyGMs[i] = pBodyGMs[i];
	timeScaling = pTimeScaling;
}

//the function to pass to the solver
void SimulationRHS(double t, double y[], double dydt[])
{
	//makes the sun stationary
	dydt[0] = 0;
	dydt[1] = 0;
	dydt[2] = 0;
	dydt[3] = 0;
	dydt[4] = 0;
	dydt[5] = 0;


	//the rate of change of position is equal to the velocity multiplied by the time scale
	for (int i = 1; i < NR_OBJECTS; i++)  {
		dydt[i * 6 + 0] = y[i * 6 + 3]*timeScaling;
		dydt[i * 6 + 1] = y[i * 6 + 4]*timeScaling;
		dydt[i * 6 + 2] = y[i * 6 + 5]*timeScaling;
	}
	
	//the rate of change of velocity is the acceleration equation (newtons grav law)
    for (int i = 1; i < NR_OBJECTS; i++)  {
		double accelX = 0.0;
		double accelY = 0.0;
		double accelZ = 0.0;

		//for every other body in the system
		for (int j = 0; j < NR_OBJECTS; j++) {
			if (j != i) {
				//calculate differences in x, y and z positions
				double diffX = y[i * 6 + 0] - y[j * 6 + 0];
				double diffY = y[i * 6 + 1] - y[j * 6 + 1];
				double diffZ = y[i * 6 + 2] - y[j * 6 + 2];		

				//use pythagoras to calculate the distance and distance squared
				double dist = sqrt( diffX*diffX + diffY*diffY + diffZ*diffZ);
				double dist2 = dist*dist;
				
				//produce normalized x, y, and z values to give the acceleration a direction
				double normalizedX = -1* diffX/dist;
				double normalizedY = -1* diffY/dist;
				double normalizedZ = -1* diffZ/dist;			

				//calculate acceleration towards other body
				accelX += (normalizedX * (bodyGMs[j] / dist2));
				accelY += (normalizedY * (bodyGMs[j] / dist2));
				accelZ += (normalizedZ * (bodyGMs[j] / dist2));
			}
		}
		//the rate of change of velocity is the acceleration multiplied by the time scale
		dydt[i * 6 + 3] = accelX*timeScaling;
		dydt[i * 6 + 4] = accelY*timeScaling;
		dydt[i * 6 + 5] = accelZ*timeScaling;
	}
}